/*****************************************************************************
IAR EWARM +SPL
Chip type               : STM32F100C4T6B
MCU Core Clock frequency: HSI 8 MHz
Project                 : 12VDC_220VAC 50Hz sinewave converter with
 ultrasonic full bridge inverter and 75_2000Hz 2 channel DCDC stage
Date                    : May, 2016
*****************************************************************************/
#include <stm32f10x.h>
#include <stm32f10x_gpio.h>
#include <stdbool.h>
#include <math.h>
#include "iostm32f10xx4.h"
#include <stm32f10x_iwdg.h>
#include "bitset.c"

#define A(x)   GPIO_WriteBit(GPIOB,GPIO_Pin_5,x)
#define B(x)   GPIO_WriteBit(GPIOB,GPIO_Pin_4,x)
#define C(x)   GPIO_WriteBit(GPIOB,GPIO_Pin_3,x)
#define D(x)   GPIO_WriteBit(GPIOA,GPIO_Pin_15,x)
#define E(x)   GPIO_WriteBit(GPIOA,GPIO_Pin_12,x)
#define F(x)   GPIO_WriteBit(GPIOA,GPIO_Pin_11,x)
#define G(x)   GPIO_WriteBit(GPIOA,GPIO_Pin_10,x)
#define DP(x)  GPIO_WriteBit(GPIOB,GPIO_Pin_15,x)
#define N0(x)  GPIO_WriteBit(GPIOB,GPIO_Pin_11|GPIO_Pin_10,x);
#define N1(x)  GPIO_WriteBit(GPIOB,GPIO_Pin_2|GPIO_Pin_1,x);
#define N2(x)  GPIO_WriteBit(GPIOB,GPIO_Pin_0,x);GPIO_WriteBit(GPIOA,GPIO_Pin_7,x)
#define N3(x)  GPIO_WriteBit(GPIOA,GPIO_Pin_6|GPIO_Pin_5,x);

#define N0_ddr(x) if(!x) { \
 GIS.GPIO_Pin = GPIO_Pin_11|GPIO_Pin_10; \
 GIS.GPIO_Mode = GPIO_Mode_AIN; \
 GPIO_Init(GPIOB, &GIS); } \
else { \
 GIS.GPIO_Pin = GPIO_Pin_11|GPIO_Pin_10; \
 GIS.GPIO_Mode = GPIO_Mode_Out_PP; \
 GPIO_Init(GPIOB, &GIS); }
#define N1_ddr(x) if(!x) { \
 GIS.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_1; \
 GIS.GPIO_Mode = GPIO_Mode_AIN; \
 GPIO_Init(GPIOB, &GIS); } \
else { \
 GIS.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_1; \
 GIS.GPIO_Mode = GPIO_Mode_Out_PP; \
 GPIO_Init(GPIOB, &GIS); }
#define N2_ddr(x) if(!x) { \
 GIS.GPIO_Pin = GPIO_Pin_0; \
 GIS.GPIO_Mode = GPIO_Mode_AIN; \
 GPIO_Init(GPIOB, &GIS); \
 GIS.GPIO_Pin = GPIO_Pin_7; \
 GPIO_Init(GPIOA, &GIS); } \
else { \
 GIS.GPIO_Pin = GPIO_Pin_0; \
 GIS.GPIO_Mode = GPIO_Mode_Out_PP; \
 GPIO_Init(GPIOB, &GIS); \
 GIS.GPIO_Pin = GPIO_Pin_7; \
 GPIO_Init(GPIOA, &GIS); }
#define N3_ddr(x) if(!x) { \
 GIS.GPIO_Pin = GPIO_Pin_6|GPIO_Pin_5; \
 GIS.GPIO_Mode = GPIO_Mode_AIN; \
 GPIO_Init(GPIOA, &GIS); } \
else { \
 GIS.GPIO_Pin = GPIO_Pin_6|GPIO_Pin_5; \
 GIS.GPIO_Mode = GPIO_Mode_Out_PP; \
 GPIO_Init(GPIOA, &GIS); }

#define cpu_freq 8000000UL
#define cnt_res 0xff
#define pwm_freq  (cpu_freq/2/cnt_res)
#define base_freq_50 50 
#define s_step pwm_freq/(base_freq_50*2)
u8 s_table [s_step];

u8 dbuf[4]={0,'_','_',0}, dpp=0;
bool ssg_pl;

GPIO_InitTypeDef GIS;

#define nok 1
enum {rls, prs, hold} btn[nok];

u32 tim[7];

const u32 r1        @0x080033e4 + sizeof(u32)*0 = 1000000 ;
const u32 r2        @0x080033e4 + sizeof(u32)*1 = 7500;
const u32 r3        @0x080033e4 + sizeof(u32)*2 = 47000;
const u32 r4        @0x080033e4 + sizeof(u32)*3 = 10000;
const u32 Vref      @0x080033e4 + sizeof(u32)*4 = 120;
const u32 CTR_rto   @0x080033e4 + sizeof(u32)*5 = 266;
const u32 r5        @0x080033e4 + sizeof(u32)*6 = 100;
const u32 Lobat_trs @0x080033e4 + sizeof(u32)*7 = 1050;
const u32 Hibat_trs @0x080033e4 + sizeof(u32)*8 = 1600;
const u32 Ktr       @0x080033e4 + sizeof(u32)*9 = 27;

#define RHV_upr r1
#define RHV_lwr r2
#define Rb_upr  r3
#define Rb_lwr  r4
#define R_ctr   r5

u16 UHV_max;
u16 ADCub310;
u16 Ub_max;
u16 Vdd;
u16 cmp_val;
u16 CTR_max;

volatile bool adc_ready;
u16  adc2_val=0;
u16  adc1_val=0;
u16  adc0_val=0;

#define adc_bat adc2_val
#define adc_hv  adc1_val
#define adc_cs  adc0_val

#define spwm_off TIM1->EGR  = TIM_EGR_BG
#define spwm_on  TIM1->BDTR|= TIM_BDTR_MOE

bool restart_bit=1;
bool t1_iwdg,stc_iwdg;

void gpio_cfg()
{
 RCC->APB2ENR |= RCC_APB2ENR_AFIOEN|RCC_APB2ENR_IOPAEN|RCC_APB2ENR_IOPBEN;

 GIS.GPIO_Speed = GPIO_Speed_2MHz;

 GPIO_WriteBit(GPIOA, GPIO_Pin_All, Bit_RESET);
 GPIO_WriteBit(GPIOB, GPIO_Pin_All, Bit_RESET);
 GPIO_WriteBit(GPIOB, GPIO_Pin_14|GPIO_Pin_13, Bit_SET); //spwm ncnls
 GPIO_WriteBit(GPIOB, GPIO_Pin_9|GPIO_Pin_8|GPIO_Pin_7|GPIO_Pin_6, Bit_SET);//fwm cnls

 //adc input pins
 GIS.GPIO_Pin   = GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_2;
 GIS.GPIO_Mode  = GPIO_Mode_AIN;
 GPIO_Init(GPIOA, &GIS);
 //7seg polarity jumper pin
 GIS.GPIO_Pin   = GPIO_Pin_3;
 GIS.GPIO_Mode  = GPIO_Mode_IPU;
 GPIO_Init(GPIOA, &GIS);
 //buzzer
 GIS.GPIO_Pin   = GPIO_Pin_4;
 GIS.GPIO_Mode  = GPIO_Mode_AIN;
 GPIO_Init(GPIOA, &GIS);
 //7seg '3,2'
 GIS.GPIO_Pin   = GPIO_Pin_5|GPIO_Pin_6|GPIO_Pin_7;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOA, &GIS);
 //7seg '2,1,0'
 GIS.GPIO_Pin   = GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_10|GPIO_Pin_11;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOB, &GIS);
 //on_off switch
 GIS.GPIO_Pin   = GPIO_Pin_12;
 GIS.GPIO_Mode  = GPIO_Mode_IPU;
 GPIO_Init(GPIOB, &GIS);
 //spwm_n outputs
 GIS.GPIO_Pin   = GPIO_Pin_14|GPIO_Pin_13;
 GIS.GPIO_Mode  = GPIO_Mode_AF_PP;
 GPIO_Init(GPIOB, &GIS);
 //7seg 'H'
 GIS.GPIO_Pin   = GPIO_Pin_15;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOB, &GIS);
 //spwm outputs
 GIS.GPIO_Pin   = GPIO_Pin_9|GPIO_Pin_8;
 GIS.GPIO_Mode  = GPIO_Mode_AF_PP;
 GPIO_Init(GPIOA, &GIS);
 //7seg 'e,f,g'
 GIS.GPIO_Pin   = GPIO_Pin_10|GPIO_Pin_11|GPIO_Pin_12;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOA, &GIS);
 //jtag remap
 GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);
 //7seg 'd'
 GIS.GPIO_Pin   = GPIO_Pin_15;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOA, &GIS);
 //7seg 'c,b,a'
 GIS.GPIO_Pin   = GPIO_Pin_3|GPIO_Pin_4|GPIO_Pin_5;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOB, &GIS);
 //low frequency 2x2 fwm
 GIS.GPIO_Pin   = GPIO_Pin_9|GPIO_Pin_8|GPIO_Pin_7|GPIO_Pin_6;
 GIS.GPIO_Mode  = GPIO_Mode_Out_PP;
 GPIO_Init(GPIOB, &GIS);

 GPIO_PinLockConfig(GPIOA, GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_9|GPIO_Pin_8);
}

void adc_cfg()
{
 RCC_APB2ENR_bit.ADC1EN = 1;

 ADC1->CR1 = ADC_CR1_SCAN;
 ADC1->CR2 = ADC_CR2_EXTTRIG|ADC_CR2_JEXTTRIG|ADC_CR2_TSVREFE|ADC_CR2_RSTCAL;

 ADC1_CR2_bit.EXTSEL  = 7; //SWSTART
 ADC1_CR2_bit.JEXTSEL = 0; //T1 TRGO

 ADC1_SQR3_bit.SQ1  = 17;  //Vref

 ADC1_JSQR_bit.JL   = 2;
 ADC1_JSQR_bit.JSQ2 = 0;
 ADC1_JSQR_bit.JSQ3 = 1;
 ADC1_JSQR_bit.JSQ4 = 2;

 ADC1_SMPR1_bit.SMP17 = 7;
 ADC1_SMPR2_bit.SMP0  = 2;
 ADC1_SMPR2_bit.SMP1  = 2;
 ADC1_SMPR2_bit.SMP2  = 2;

 ADC1_CR2_bit.ADON = 1;

 for(volatile u32 i=5E5;i;i--);
 ADC1->CR2|=ADC_CR2_CAL;
 while(ADC1_CR2_bit.CAL);
 ADC1->SR = 0;
 ADC1_CR2_bit.SWSTART = 1;
 while(!ADC1_SR_bit.EOC);
}

void t1_cfg()
{
 RCC->APB2ENR |= RCC_APB2ENR_TIM1EN;

 TIM1->ARR  = 0xff;
 TIM1->PSC  = 0;
 TIM1->CCR1 = 0;
 TIM1->CCR2 = 0;
 TIM1->CCR4 = 0xfe;
 TIM1->CR1  = TIM_CR1_CMS_0|TIM_CR1_URS; //center aligned mode 1
 TIM1->CR2  = TIM_CR2_OIS2|TIM_CR2_OIS2N|TIM_CR2_OIS1|TIM_CR2_OIS1N;
 TIM1_CR2_bit.MMS = 7; //OC4ref as TRGO
 TIM1->CCER = TIM_CCER_CC1E|TIM_CCER_CC1NE|TIM_CCER_CC2E|TIM_CCER_CC2NE;
 TIM1->CCER|= TIM_CCER_CC1NP|TIM_CCER_CC2NP;
 TIM1->RCR  = 1;
 TIM1->DIER = TIM_DIER_CC4IE;
 TIM1->BDTR = TIM_BDTR_OSSI|24; // dt delay (*125ns)

 TIM1->CCMR1 = TIM_CCMR1_OC2M_2|TIM_CCMR1_OC1M_1|TIM_CCMR1_OC1M_2|TIM_CCMR1_OC1PE|TIM_CCMR1_OC2PE;
 TIM1_CCMR2_bit.OC4M = 6; //PWM_1

 TIM1->BDTR |= TIM_BDTR_LOCK_1;

 NVIC_SetPriority(TIM1_CC_IRQn,1);
 NVIC_EnableIRQ(TIM1_CC_IRQn);
 TIM1->CR1  |= TIM_CR1_CEN;
}
void t16_cfg()
{
 RCC->APB2ENR |= RCC_APB2ENR_TIM16EN;

 TIM16->CNT    = 0;
 TIM16->PSC    = 3;
 TIM16->ARR    = (u16)((u32)8E6/200);
 TIM16->CCMR1  = TIM_CCMR1_OC1M_1|TIM_CCMR1_OC1M_2;//pwm mode 1
 TIM16->CCER   = TIM_CCER_CC1E|TIM_CCER_CC1NE;
 TIM16->CCER  |= TIM_CCER_CC1P|TIM_CCER_CC1NP;
 TIM16->CCR1   = TIM16->ARR/2;
 TIM16->CR2    = TIM_CR2_OIS1|TIM_CR2_OIS1N;
 TIM16->BDTR   = TIM_BDTR_OSSI|40;

 TIM16->CR1    = TIM_CR1_CKD_0|TIM_CR1_ARPE;
 TIM16->EGR    = TIM_EGR_COMG|TIM_EGR_UG;
 TIM16->BDTR  |= TIM_BDTR_LOCK_1|TIM_BDTR_LOCK_0;

 GIS.GPIO_Pin   = GPIO_Pin_8|GPIO_Pin_6;
 GIS.GPIO_Mode  = GPIO_Mode_AF_PP;
 GPIO_Init(GPIOB, &GIS);
}
void t17_cfg()
{
 RCC->APB2ENR |= RCC_APB2ENR_TIM17EN;

 TIM17->CNT    = 0;
 TIM17->PSC    = 3;
 TIM17->ARR    = (u16)((u32)8E6/200);
 TIM17->CCMR1  = TIM_CCMR1_OC1M_1|TIM_CCMR1_OC1M_2;//pwm mode 1
 TIM17->CCER   = TIM_CCER_CC1E|TIM_CCER_CC1NE;
 TIM17->CCER  |= TIM_CCER_CC1P|TIM_CCER_CC1NP;
 TIM17->CCR1   = TIM17->ARR/2;
 TIM17->CR2    = TIM_CR2_OIS1|TIM_CR2_OIS1N;
 TIM17->BDTR   = TIM_BDTR_OSSI|40;

 TIM17->CR1    = TIM_CR1_CKD_0|TIM_CR1_ARPE;
 TIM17->EGR    = TIM_EGR_COMG|TIM_EGR_UG;
 TIM17->BDTR  |= TIM_BDTR_LOCK_1|TIM_BDTR_LOCK_0;

 GIS.GPIO_Pin   = GPIO_Pin_9|GPIO_Pin_7;
 GIS.GPIO_Mode  = GPIO_Mode_AF_PP;
 GPIO_Init(GPIOB, &GIS);
}
void systic_cfg(u32 frq)
{
 SysTick->LOAD  = (u32)8E6/frq-1;
 NVIC_SetPriority (SysTick_IRQn, 2);
 SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk|SysTick_CTRL_TICKINT_Msk|SysTick_CTRL_ENABLE_Msk;
}
void buzzer_cfg()
{
#define DAC_DHR8R1_ADDRESS 0x40007410UL

 static u8 tab[2]={0,0xff};

 RCC->AHBENR         |= RCC_AHBENR_DMA1EN;
 DMA1_Channel3->CCR   = DMA_CCR3_CIRC;
 DMA1_Channel3->CCR  |= DMA_CCR3_DIR;
 DMA1_Channel3->CCR  |= DMA_CCR3_MINC;
 DMA1_Channel3->CNDTR = 2;
 DMA1_Channel3->CPAR  = DAC_DHR8R1_ADDRESS;
 DMA1_Channel3->CMAR  = (u32)&tab[0];
 DMA1_Channel3->CCR  |= DMA_CCR3_EN;

 RCC->APB1ENR |= RCC_APB1ENR_TIM6EN;
 TIM6->PSC     = 0;
 TIM6->ARR     = cpu_freq/8E3;
 TIM6->CR2     = TIM_CR2_MMS_1;
 TIM6->DIER    = TIM_DIER_UDE;

 RCC->APB1ENR |= RCC_APB1ENR_DACEN;
 DAC->CR      |= DAC_CR_BOFF1;
 DAC->CR      |= DAC_CR_DMAEN1;
 DAC->CR      |= DAC_CR_EN1;
}

void iwdg_cfg()
{
 IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
 IWDG_SetPrescaler(IWDG_Prescaler_128);
 IWDG_SetReload(0xfff/2);
 IWDG_Enable();
 IWDG_ReloadCounter();
}

void detect_7seg_polarity()
{
 u8 n=0;
 for(u8 k=0;k<0xff;k++)
 {
  if(GPIOA_IDR_bit.IDR3) n++;
  else if(n)n--;
 }

 if(n>0xff/2)ssg_pl=0; //CA
 else ssg_pl=1; //CC

 GIS.GPIO_Pin   = GPIO_Pin_3;
 GIS.GPIO_Mode  = GPIO_Mode_Out_OD;
 GPIO_Init(GPIOA, &GIS);
 GPIO_WriteBit(GPIOA, GPIO_Pin_3, Bit_RESET);
}
void convert_data (u16 d)
{
 for (u8 k=0;k<4;k++)
  {
   dbuf[k]=d%10;
   d/=10;
  }
}
void display_data()
{
 static u8 pc; u8 tmp;

 N3_ddr(0);N2_ddr(0);N1_ddr(0);N0_ddr(0);

 (pc)?(pc--):(pc = 3);

 switch (dbuf[pc])
 {
  case 0: tmp = (u8)bin(0000001); break;
  case 1: tmp = (u8)bin(1001111); break;
  case 2: tmp = (u8)bin(0010010); break;
  case 3: tmp = (u8)bin(0000110); break;
  case 4: tmp = (u8)bin(1001100); break;
  case 5: tmp = (u8)bin(0100100); break;
  case 6: tmp = (u8)bin(0100000); break;
  case 7: tmp = (u8)bin(0001111); break;
  case 8: tmp = (u8)bin(0000000); break;
  case 9: tmp = (u8)bin(0000100); break;

  case 10: tmp = (u8)bin(0111111); break;
  case 11: tmp = (u8)bin(1011111); break;
  case 12: tmp = (u8)bin(1101111); break;
  case 13: tmp = (u8)bin(1110111); break;
  case 14: tmp = (u8)bin(1111011); break;
  case 15: tmp = (u8)bin(1111101); break;
  case 16: tmp = (u8)bin(1111110); break;

  case ' ': tmp = (u8)bin(1111111); break;
  case 'v':
  case 'u': tmp = (u8)bin(1100011); break;
  case 'c': tmp = (u8)bin(1110010); break;
  case 'b': tmp = (u8)bin(1100000); break;
  case 't': tmp = (u8)bin(1110000); break;
  case 'l': tmp = (u8)bin(1110011); break;
  case 'o': tmp = (u8)bin(1100010); break;
  case 'A': tmp = (u8)bin(0001000); break;
  case 'H': tmp = (u8)bin(1001000); break;
  case 'C': tmp = (u8)bin(0110001); break;
  case 'P': tmp = (u8)bin(0011000); break;
  case 'E': tmp = (u8)bin(0110000); break;
  case 'U': tmp = (u8)bin(1000001); break;
  case 'F': tmp = (u8)bin(0111000); break;
  case 'L': tmp = (u8)bin(1110001); break;
  case 'r': tmp = (u8)bin(1111010); break;
  case '_': tmp = (u8)bin(1110111); break;
  case 'd': tmp = (u8)bin(1000010); break;
  case 'n': tmp = (u8)bin(1101010); break;
  case 'G': tmp = (u8)bin(0100001); break;
  case 'N': tmp = (u8)bin(0001001); break;
  case 'O': tmp = (u8)bin(0000001); break;
  case 'i': tmp = (u8)bin(1111011); break;
  case 'I': tmp = (u8)bin(1111001); break;
  case 'S': tmp = (u8)bin(0100100); break;
  case 'J': tmp = (u8)bin(1000111); break;
  case 'h': tmp = (u8)bin(1101000); break;
  default:  tmp = (u8)bin(1111110); break; //-
 }

 if(ssg_pl)//CC
 {
  tmp = ~tmp;

  DP(0);
  N3(1);N2(1);N1(1);N0(1);

  switch (pc)
  {
   case 0: N0(0); if(dpp == 0) {DP(1);} break;
   case 1: N1(0); if(dpp == 1) {DP(1);} break;
   case 2: N2(0); if(dpp == 2) {DP(1);} break;
   case 3: if (dbuf[3])N3(0); if(dpp == 3) {DP(1);} break;
  }
 }
 else //CA
 {
  DP(1);
  N3(0);N2(0);N1(0);N0(0);

  switch (pc)
  {
   case 0: N0(1); if(dpp == 0) {DP(0);} break;
   case 1: N1(1); if(dpp == 1) {DP(0);} break;
   case 2: N2(1); if(dpp == 2) {DP(0);} break;
   case 3: if (dbuf[3])N3(1); if(dpp == 3) {DP(0);} break;
  }
 }

 if(tmp&(1<<6))A(1);else A(0);
 if(tmp&(1<<5))B(1);else B(0);
 if(tmp&(1<<4))C(1);else C(0);
 if(tmp&(1<<3))D(1);else D(0);
 if(tmp&(1<<2))E(1);else E(0);
 if(tmp&(1<<1))F(1);else F(0);
 if(tmp&(1<<0))G(1);else G(0);

 N3_ddr(1);N2_ddr(1);N1_ddr(1);N0_ddr(1);
}
void scan_keys ()
{
#define lp  25
#define lh  2*500/lp
#define lr  10

static u8 count[nok*3];
u8 key_st[nok];

key_st[0] = GPIOB_IDR_bit.IDR12;


 for(u8 i=0;i<nok;i++)
 {
  if (!key_st[i]) {count[i*3]++;count[i*3+2]=lr;}
  else if((key_st[i])&&(!count[i*3+2])) {count[i*3]=count[i*3+1]=0;btn[i]=rls;}
  else count[i*3+2]--;

  if ((count[i*3]>lp)&&(btn[i]!=hold)) {btn[i]=prs; count[i*3]=0; count[i*3+1]++;}
  if (count[i*3+1]>lh) {btn[i]=hold; count[i*3+1]=0;}
 }
}

void TIM1_CC_IRQHandler ()
{
 static u16 mst;
 u16 tmp_table;
 static u8 i=0;
 static u16 adc_cnt=0;
 static u32 adc0_tmp=0, adc1_tmp=0, adc2_tmp=0;
 u16 adc_tmp;

 TIM1->SR = 0;

 adc0_tmp+= ADC1->JDR1;
 adc1_tmp+= adc_tmp = ADC1->JDR2;
 adc2_tmp+= ADC1->JDR3;
 adc_cnt++;

 if(restart_bit){restart_bit=0;mst=0;}

 if(i>s_step-1)
 {
  static bool adc_trg=0;

  i=0;

  if(TIM1->CCMR1 & TIM_CCMR1_OC2M_1)
   TIM1->CCMR1 = TIM_CCMR1_OC2M_2|TIM_CCMR1_OC1M_2|TIM_CCMR1_OC1M_1|TIM_CCMR1_OC1PE|TIM_CCMR1_OC2PE;
  else
   TIM1->CCMR1 = TIM_CCMR1_OC1M_2|TIM_CCMR1_OC2M_2|TIM_CCMR1_OC2M_1|TIM_CCMR1_OC1PE|TIM_CCMR1_OC2PE;

  if(!adc_trg) adc_trg=1;
  else
  {
   adc_trg=0;
   adc0_val = adc0_tmp/adc_cnt;
   adc1_val = adc1_tmp/adc_cnt;
   adc2_val = adc2_tmp/adc_cnt;
   adc0_tmp = adc1_tmp = adc2_tmp = adc_cnt = 0;
   adc_ready = 1;
  }

  if(mst<0xff/2)mst+=5;
 }

 {
  if(mst<0xff/2);
  else if(adc_tmp>cmp_val) mst=(u16)((u32)ADCub310*0xff/adc_tmp);
  else 	mst=0x1ff;

  tmp_table=(u16)((u32)s_table [i]*mst>>8);
  if(tmp_table>0xff) tmp_table=0xff;

  TIM1->CCR1=TIM1->CCR2=(u8)tmp_table;
  i++;
 }
 t1_iwdg   = 1;
}

void SysTick_Handler()
{

 display_data();
 scan_keys();

 if(tim[0])tim[0]--;
 if(tim[1])tim[1]--;
 if(tim[2])tim[2]--;
 if(tim[3])tim[3]--;
 if(tim[4])tim[4]--;
 //if(tim[5])tim[5]--;
 if(tim[6])tim[6]--; else {TIM6->CR1&=~TIM_CR1_CEN;}// DAC->DHR8R1=0;}
 stc_iwdg = 1;
}

void set_dcdc(bool, bool, u16);
void beep(u16 freq, u16 time_ms);
void devcon();

void main()
{
 gpio_cfg();
 adc_cfg();
 detect_7seg_polarity();

 for(u16 i=0;i<s_step;i++) s_table[i]=(u8)(cnt_res*sin(3.1416*i/(s_step-1)));

 Vdd      = (u16)((u32)Vref*0xfff/ADC1->DR);           //~330 (V*100)
 UHV_max  = (u16)((u32)Vdd*(RHV_upr+RHV_lwr)/RHV_lwr); //~44300 (V*100)
 ADCub310 = (u16)((u32)31000*0xfff/UHV_max);           //~2861 (adc_pts)
 cmp_val  = (u16)(20000UL*0xfff/UHV_max);              //~1847 (adc_pts)
 Ub_max   = (u16)(((u32)Vdd*(Rb_upr+Rb_lwr))/Rb_lwr);  //~1881 (V*100)
 CTR_max  = (u16)((u32)Vdd*CTR_rto/R_ctr);             //~877  (A*100)

 t1_cfg();
 t16_cfg();
 t17_cfg();
 GPIO_PinLockConfig(GPIOB, GPIO_Pin_14|GPIO_Pin_13|GPIO_Pin_9|GPIO_Pin_8|GPIO_Pin_7|GPIO_Pin_6);
 systic_cfg(500);
 buzzer_cfg();
 iwdg_cfg();

 while(1)
 {
  devcon();

  if(t1_iwdg && stc_iwdg){t1_iwdg=stc_iwdg=0; IWDG_ReloadCounter();}
 }
}

void devcon()
{
 static enum  {start,work,LoHV,LoBat,EHBat,off,ocp} dev_st=start;
 static bool itr=0, btn_trg=0;
 u32 tmp;

 if(btn[0]==rls){btn_trg=0;}

 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;
    set_dcdc(0, 0, 1000);
    spwm_off;
    adc_ready=0;
    while(!adc_ready);
    dpp=1;
    convert_data((u32)adc_bat*Ub_max/0xfff/10);
    tim[0]=1000;
    while(tim[0]);

    if((u32)adc_bat*Ub_max/0xfff<Lobat_trs)      {dev_st = LoBat; goto m;}
    else if((u32)adc_bat*Ub_max/0xfff>Hibat_trs) {dev_st = EHBat; goto m;}
    tim[1]=2000;
    set_dcdc(1, 0, 2000);
    tim[0]=100;
    beep(4000,300);
   }

   if(!tim[0])
   {
    static u8 k=0;
    tim[0]=100;

    if(!k)       dbuf[3]=dbuf[2]=dbuf[1]=' ',dbuf[0]=13,dpp=9;
    else if(k==1)dbuf[3]=dbuf[2]=dbuf[1]=' ',dbuf[0]=16;
    else         dbuf[3]=dbuf[2]=dbuf[1]=' ',dbuf[0]=10;
    if(k<2)k++; else k=0;

    if((u32)adc_hv*UHV_max/0xfff>28000)dev_st = work;
    else if(!tim[1])dev_st = LoHV;
   }

   m:
	 if(dev_st!=start){itr=0;}
	}
  break;

  case work:
  {
   static enum {Lvolts, Hvolts, amps, pow} db_state = Lvolts;
   static u32 trs_1A;

   if(!itr)
   {
		itr=1;
    trs_1A = 100UL*0xfff/CTR_max;
    restart_bit=1;
    set_dcdc(1, 1, 200);
    spwm_on;
    tim[1]=10*500;
    beep(3000,300);
   }

//dc_dc conrtol section
   {
    if(adc_ready)
    {
     static u8 stp = 0;

     adc_ready = 0;
     tmp = (u32)adc_hv*UHV_max/0xfff;

     if(tmp>43500){set_dcdc(0, 0, 200); stp=0;}
     else if (tmp<42500)
     {
      if(adc_cs<trs_1A)
      {
       if(!tim[1] && stp!=1){set_dcdc(1, 0, 200); stp=1;}
      }
      else if(adc_cs<trs_1A*2)
      {
       if(stp<2)
       {set_dcdc(1, 0, 100); tim[1]=60*500; stp=2;}
       else if(!tim[1]){set_dcdc(1, 0, 100); tim[1]=60*500; stp=2;}
      }
      else if(adc_cs<trs_1A*3)
      {
       if(stp<3)
       {set_dcdc(1, 1, 200); tim[1]=60*500; stp=3;}
       else if(!tim[1]){set_dcdc(1, 1, 200); tim[1]=60*500; stp=3;}
      }
      else if(adc_cs<trs_1A*4)
      {
       if(stp<4)
       {set_dcdc(1, 1, 150); tim[1]=60*500; stp=4;}
       else if(!tim[1]){set_dcdc(1, 1, 150); tim[1]=60*500; stp=4;}
      }
      else if(adc_cs<trs_1A*5)
      {
       if(stp<5)
       {set_dcdc(1, 1, 100); tim[1]=60*500; stp=5;}
       else if(!tim[1]){set_dcdc(1, 1, 100); tim[1]=60*500; stp=5;}
      }
      else if(adc_cs<trs_1A*6)
      {
       if(stp<6)
       {set_dcdc(1, 1, 75); tim[1]=10*500; stp=6;}
       else if(!tim[1]){set_dcdc(1, 1, 75); tim[1]=60*500; stp=6;}
      }
      else
      {
       if(stp!=7)
       {set_dcdc(1, 1, 75); tim[4]=10*500; stp=7;}
       else if(!tim[4]) dev_st = ocp;
      }
     }
    }
   }
//end of dc_dc control section

//keyworks
   {
    static bool bp_trg=0;

    if(btn[0]==hold&&!btn_trg){btn_trg=1;bp_trg=0;dev_st=off;}
    else if(btn[0]==prs&&!bp_trg)bp_trg=1;

    if(btn[0]==rls && bp_trg)
    {
     bp_trg=0;
     beep(5000,100);
     if(db_state==Lvolts)      {db_state = Hvolts;}
     else if(db_state==Hvolts) {db_state = amps;  }
     else if(db_state==amps)   {db_state = pow;   }
     else if(db_state==pow)    {db_state = Lvolts;}
     else                      {db_state = Lvolts;}
    }
   }
//end of keyworks

    if(!tim[0])
   {
    tim[0]=500;
//displayworks
    {
     if(db_state==Lvolts)
     {convert_data((u32)adc_bat*Ub_max/0xfff/10);       dpp=1; dbuf[3]='L';}
     else if(db_state==Hvolts)
     {convert_data((u32)adc_hv*UHV_max/0xfff/100);      dpp=3; dbuf[3]='H';}
     else if(db_state==amps)
     {convert_data((u32)adc_cs*CTR_max/0xfff);          dpp=2; dbuf[3]='A';}
     else if(db_state==pow)
     {convert_data((u32)220*adc_cs*CTR_max/0xfff/100);  dpp=9;}
    }
//end of displayworks

//excessive prms protection section
    {
     static u8 bt_cnt=0, hv_cnt=0;

     tmp = (u32)adc_bat*Ub_max/0xfff;

     if(tmp<900) dev_st=LoBat;
     else if(tmp<Lobat_trs && bt_cnt<10) {bt_cnt++; beep(1000,300);}
     else if(tmp<Lobat_trs) dev_st=LoBat;
     else bt_cnt=0;

     if((u32)adc_hv<cmp_val && hv_cnt<1) {hv_cnt++; beep(5000,300);}
     else if((u32)adc_hv<cmp_val) dev_st=LoHV;
     else hv_cnt=0;
    }
//end of exc prms protection section
   }

	 if(dev_st!=work){itr=0;}
  }
  break;

 case LoBat:
  {
	 if(!itr)
   {
		itr=1;
    set_dcdc(0, 0, 400);
    spwm_off;
    tim[0]=10*500;
    beep(3000,1000);
    dbuf[3]='L';dbuf[2]='o';dbuf[1]='b';dbuf[0]='t'; dpp=2;
   }

   if(!tim[0])
   {
    static bool trg=0;

    tim[0]=4*500;

    if((u32)adc_bat*Ub_max/0xfff>Lobat_trs+150) dev_st=start;
    else if(!trg)
    {
     trg=1;
     convert_data((u32)adc_bat*Ub_max/0xfff/10); dpp=1; dbuf[3]='L';
    }
    else
    {
     trg=0;
     dbuf[2]='o';dbuf[1]='b';dbuf[0]='t'; dpp=2;
     beep(1000,1000);
    }
   }



	if(dev_st!=LoBat){itr=0;}
  }
  break;

 case EHBat:
  {
	 if(!itr)
   {
		itr=1;
    set_dcdc(0, 0, 400);
    spwm_off;
    tim[0]=10*500;
    beep(2000,1000);
    dbuf[3]='E';dbuf[2]='H';dbuf[1]='b';dbuf[0]='t'; dpp=2;
   }

   if(!tim[0])
   {
    static bool trg=0;

    tim[0]=4*500;

    if((u32)adc_bat*Ub_max/0xfff<Hibat_trs) dev_st=start;
    else if(!trg)
    {
     trg=1;
     convert_data((u32)adc_bat*Ub_max/0xfff/10); dpp=1; dbuf[3]='b';
    }
    else
    {
     trg=0;
     dbuf[2]='E'; dbuf[2]='H';dbuf[1]='b';dbuf[0]='t'; dpp=2;
     beep(2000,1000);
    }
   }

	 if(dev_st!=EHBat){itr=0;}
  }
  break;

 case off:
  {
	 if(!itr)
   {
		itr=1;
    set_dcdc(0, 0, 400);
    spwm_off;
    beep(2500,1000);
		dbuf[3]=dbuf[2]=dbuf[1]=dbuf[0]='_'; dpp=5;
   }

   if(btn[0]==prs&&!btn_trg){btn_trg=1;dev_st=start;}


	if(dev_st!=off){itr=0;}
  }
  break;

  case LoHV:
  {
	 if(!itr)
   {
		itr=1;
    set_dcdc(0, 0, 400);
    spwm_off;
    beep(3000,1000);
		dbuf[3]='L'; dbuf[2]='o'; dbuf[1]='H'; dbuf[0]=' '; dpp=2;
		tim[0]=5*500;
   }
   static u8 lohv_cnt=0;

   if(lohv_cnt<5)
   {
    if(!tim[2]) lohv_cnt=0;
    if(!tim[0]){lohv_cnt++; tim[2]=5*60*500; dev_st=start;}
    dbuf[0]=lohv_cnt;
   }
   else {dbuf[3]='L'; dbuf[2]='o'; dbuf[1]='c'; dbuf[0]='d'; dpp=9;}

   if(!tim[0])if(btn[0]==prs&&!btn_trg){btn_trg=1;dev_st=start;}


  if(dev_st!=LoHV){itr=0;}
  }
  break;

 case ocp:
  {
	 if(!itr)
   {
		itr=1;
    set_dcdc(0, 0, 400);
    spwm_off;
    beep(4000,1000);
    tim[0]=10*500;
		dbuf[2]='O'; dbuf[1]='C'; dbuf[0]='P'; dpp=9;
   }
   static u8 ocp_cnt=0;

   if(ocp_cnt<5)
   {
    if(!tim[3]) ocp_cnt=0;
    if(!tim[0]){ocp_cnt++; tim[3]=5*60*500; dev_st=start;}
    dbuf[0]=ocp_cnt;
   }
   else {dbuf[3]='L'; dbuf[2]='o'; dbuf[1]='c'; dbuf[0]='C'; dpp=9;}

   if(!tim[0])if(btn[0]==prs&&!btn_trg){btn_trg=1;dev_st=start;}


	 if(dev_st!=off){itr=0;}
  }
  break;

 default: break;
 }
}

void set_dcdc(bool CNL1, bool CNL2, u16 frq)
{
 static bool CNL_1=0, CNL_2=0;
 static u16 dcdc_frq=400;

 if(frq<75)frq=75; else if(frq>2000)frq=2000;
 if(CNL_1!=CNL1 || CNL_2!=CNL2 || dcdc_frq!=frq)
 {
  CNL_1=CNL1;
  CNL_2=CNL2;
  dcdc_frq=frq;

  if(CNL1 && CNL2)
  {
   TIM17->EGR  = TIM_EGR_BG;
   TIM16->EGR  = TIM_EGR_BG;
   while (TIM16->BDTR&TIM_BDTR_MOE);
   TIM17->CR1  &= ~TIM_CR1_CEN;
   TIM16->CR1  &= ~TIM_CR1_CEN;

   TIM17->ARR  = TIM16->ARR   = cpu_freq/4/frq;
   TIM17->CCR1 = TIM16->CCR1  = TIM16->ARR/2;
   TIM17->CNT  = 0;
   TIM16->CNT  = TIM16->ARR/4;

   TIM17->EGR   = TIM_EGR_UG;
   TIM16->EGR   = TIM_EGR_UG;
   TIM17->CR1  |= TIM_CR1_CEN;
   TIM16->CR1  |= TIM_CR1_CEN;
   TIM17->BDTR |= TIM_BDTR_MOE;
   TIM16->BDTR |= TIM_BDTR_MOE;
  }
  else if(CNL1)
  {
   TIM17->EGR  = TIM_EGR_BG;
   TIM16->EGR  = TIM_EGR_BG;
   while (TIM16->BDTR&TIM_BDTR_MOE);
   TIM17->CR1  &= ~TIM_CR1_CEN;
   TIM16->CR1  &= ~TIM_CR1_CEN;

   TIM17->ARR   = cpu_freq/4/frq;
   TIM17->CCR1  = TIM17->ARR/2;
   TIM17->CNT   = 0;

   TIM17->EGR   = TIM_EGR_UG;
   TIM17->CR1  |= TIM_CR1_CEN;
   TIM17->BDTR |= TIM_BDTR_MOE;
  }
  else
  {
   TIM17->EGR  = TIM_EGR_BG;
   TIM16->EGR  = TIM_EGR_BG;
  }
 }

}

void beep(u16 freq, u16 time_ms)
{
 TIM6->ARR = (u16)(cpu_freq/2/freq);
 tim[6]    = time_ms/2;
 TIM6->CR1 = TIM_CR1_CEN;
}